@@ -22,13 +22,17 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
 #include "libgossip-marshal.h"
-#include "gossip-utils.h"
 #include "gossip-account-manager.h"
+#include "gossip-debug.h"
+#include "gossip-utils.h"
+
+#define DEBUG_DOMAIN "AccountManager"
+
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_ACCOUNT_MANAGER, GossipAccountManagerPriv))
 
 /* For splitting an id of user@server/resource to just user@server with resource
  * in it's own xml tags (for Gossip release 0.10.2).
@@ -38,11 +42,6 @@
 #define ACCOUNTS_XML_FILENAME "accounts.xml"
 #define ACCOUNTS_DTD_FILENAME "gossip-account.dtd"
 
-#define DEBUG_MSG(x)
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
-
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_ACCOUNT_MANAGER, GossipAccountManagerPriv))
-
 typedef struct _GossipAccountManagerPriv GossipAccountManagerPriv;
 
 struct _GossipAccountManagerPriv {
@@ -61,16 +60,20 @@ enum {
 	LAST_SIGNAL
 };
 
-static void     account_manager_finalize   (GObject              *object);
-static gboolean account_manager_get_all    (GossipAccountManager *manager);
-static gboolean account_manager_file_parse (GossipAccountManager *manager,
-					    const gchar          *filename);
-static gboolean account_manager_file_save  (GossipAccountManager *manager);
+static void     account_manager_finalize               (GObject              *object);
+static gboolean account_manager_get_all                (GossipAccountManager *manager);
+static gboolean account_manager_file_parse             (GossipAccountManager *manager,
+							const gchar          *filename);
+static gboolean account_manager_file_save              (GossipAccountManager *manager);
+static void     account_manager_add_parameters_foreach (GossipAccount        *account,
+							gchar                *param_name,
+							GossipAccountParam   *param,
+							xmlNodePtr            node);
 
 static guint signals[LAST_SIGNAL] = {0};
 
 #ifdef RESOURCE_HACK
-gboolean need_saving = FALSE;
+static gboolean need_saving = FALSE;
 #endif
 
 G_DEFINE_TYPE (GossipAccountManager, gossip_account_manager, G_TYPE_OBJECT);
@@ -84,26 +87,26 @@ gossip_account_manager_class_init (GossipAccountManagerClass *klass)
 
 	signals[ACCOUNT_ADDED] = 
 		g_signal_new ("account-added",
-                              G_TYPE_FROM_CLASS (klass),
+			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
-                              NULL, NULL,
+			      NULL, NULL,
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_ACCOUNT);
-        signals[ACCOUNT_REMOVED] = 
+	signals[ACCOUNT_REMOVED] = 
 		g_signal_new ("account-removed",
 			      G_TYPE_FROM_CLASS (klass),
-                              G_SIGNAL_RUN_LAST,
+			      G_SIGNAL_RUN_LAST,
 			      0, 
 			      NULL, NULL,
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_ACCOUNT);
-        signals[NEW_DEFAULT] = 
+	signals[NEW_DEFAULT] = 
 		g_signal_new ("new-default",
 			      G_TYPE_FROM_CLASS (klass),
-                              G_SIGNAL_RUN_LAST,
+			      G_SIGNAL_RUN_LAST,
 			      0, 
 			      NULL, NULL,
 			      libgossip_marshal_VOID__OBJECT,
@@ -126,7 +129,7 @@ account_manager_finalize (GObject *object)
 	
 	priv = GET_PRIV (object);
 
-	g_list_foreach (priv->accounts, (GFunc)g_object_unref, NULL);
+	g_list_foreach (priv->accounts, (GFunc) g_object_unref, NULL);
 	g_list_free (priv->accounts);
 
 	g_free (priv->accounts_file_name);
@@ -158,7 +161,7 @@ gossip_account_manager_new (const gchar *filename)
 	return manager;
 }
 
-gboolean          
+gboolean
 gossip_account_manager_add (GossipAccountManager *manager,
 			    GossipAccount        *account)
 {
@@ -177,9 +180,9 @@ gossip_account_manager_add (GossipAccountManager *manager,
 		type = gossip_account_get_type (account);
 		name = gossip_account_get_name (account);
 
-		DEBUG_MSG (("Account Manager: Adding %s account with name:'%s'", 
-			   gossip_account_type_to_string (type), 
-			   name));
+		gossip_debug (DEBUG_DOMAIN, "Adding %s account with name:'%s'", 
+			      gossip_account_type_to_string (type), 
+			      name);
 		
 		priv->accounts = g_list_append (priv->accounts, g_object_ref (account));
 
@@ -191,7 +194,7 @@ gossip_account_manager_add (GossipAccountManager *manager,
 	return FALSE;
 }
 
-void          
+void
 gossip_account_manager_remove (GossipAccountManager *manager,
 			       GossipAccount        *account)
 {
@@ -202,8 +205,9 @@ gossip_account_manager_remove (GossipAccountManager *manager,
 
 	priv = GET_PRIV (manager);
 
- 	DEBUG_MSG (("Account Manager: Removing account with name:'%s'",  
- 		   gossip_account_get_name (account)));
+ 	gossip_debug (DEBUG_DOMAIN, 
+		      "Removing account with name:'%s'",  
+		      gossip_account_get_name (account));
 
 	priv->accounts = g_list_remove (priv->accounts, account);
 	
@@ -306,8 +310,9 @@ gossip_account_manager_set_overridden_default (GossipAccountManager *manager,
 
 	priv = GET_PRIV (manager);
 
- 	DEBUG_MSG (("Account Manager: Setting overriding default account with name:'%s'",  
- 		   name)); 
+ 	gossip_debug (DEBUG_DOMAIN, 
+		      "Setting overriding default account with name:'%s'",  
+		      name); 
 
 	g_free (priv->default_name_override);
 	priv->default_name_override = g_strdup (name);
@@ -325,8 +330,9 @@ gossip_account_manager_set_default (GossipAccountManager *manager,
 
 	priv = GET_PRIV (manager);
 	
- 	DEBUG_MSG (("Account Manager: Setting default account with name:'%s'",  
- 		   gossip_account_get_name (account))); 
+ 	gossip_debug (DEBUG_DOMAIN, 
+		      "Setting default account with name:'%s'",  
+		      gossip_account_get_name (account)); 
 
 	name = gossip_account_get_name (account);
 	g_return_if_fail (name != NULL);
@@ -370,14 +376,15 @@ gossip_account_manager_get_default (GossipAccountManager *manager)
 			GList         *l;
 
 			l = gossip_account_manager_get_accounts (manager);
-			account = g_object_ref (l->data);
-
-			g_list_foreach (l, (GFunc) g_object_unref, NULL);
-			g_list_free (l);
+			account = l->data;
 
 			name = gossip_account_get_name (account);
 			gossip_account_manager_set_default (manager, account);
 
+			g_list_foreach (l, (GFunc) g_object_unref, NULL);
+			g_list_free (l);
+
+			/* We return the default account ref here. */
 			return account;
 		}
 	} else {
@@ -392,7 +399,7 @@ gossip_account_manager_store (GossipAccountManager *manager)
 {
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), FALSE);
 
- 	DEBUG_MSG (("Account Manager: Saving accounts"));  
+ 	gossip_debug (DEBUG_DOMAIN, "Saving accounts");  
 	
 	return account_manager_file_save (manager);
 }
@@ -439,26 +446,16 @@ static void
 account_manager_parse_account (GossipAccountManager *manager,
 			       xmlNodePtr            node)
 {
-	GossipAccount     *account;
-	xmlNodePtr         child;
-	gchar             *str;
-	GossipAccountType  type;
-	gchar             *name, *id, *password;
-	gchar             *server, *resource;
-	guint16            port;
-	gboolean           auto_connect, use_ssl, use_proxy;
+	GossipAccount *account;
+	xmlNodePtr     child;
+	gchar         *str;
 
-	/* Default values. */
-	type = GOSSIP_ACCOUNT_TYPE_JABBER;
-	name = NULL;
-	id = NULL;
-	password = NULL;
-	resource = NULL;
-	server = NULL;
-	port = 5222;
-	auto_connect = TRUE;
-	use_ssl = FALSE;
-	use_proxy = FALSE;
+	/* Default values */
+	account = g_object_new (GOSSIP_TYPE_ACCOUNT,
+				"type", GOSSIP_ACCOUNT_TYPE_JABBER,
+				"auto_connect", TRUE,
+				"port", 5222,
+				NULL);
 
 	child = node->children;
 	while (child) {
@@ -467,121 +464,119 @@ account_manager_parse_account (GossipAccountManager *manager,
 		tag = (gchar *) child->name;
 		str = (gchar *) xmlNodeGetContent (child);
 
-		if (strcmp (tag, "type") == 0) {
-			if (strcmp (str, "jabber") == 0) {
-				type = GOSSIP_ACCOUNT_TYPE_JABBER;
+		if (strcmp (tag, "name") == 0) {
+			gossip_account_set_name (account, str);
+		}
+		else if (strcmp (tag, "auto_connect") == 0) {
+			gossip_account_set_auto_connect (account, strcmp (str, "yes") == 0);
+		}
+		else if (strcmp (tag, "use_proxy") == 0) {
+			gossip_account_set_use_proxy (account, strcmp (str, "yes") == 0);
+		}
+		else if (strcmp (tag, "type") == 0) {
+			g_object_set (account,
+				      "type",
+				      gossip_account_string_to_type (str),
+				      NULL);
+		}
+		else if (strcmp (tag, "parameter") == 0) {
+			GValue                  *g_value;
+			gchar                   *param_name;
+			gchar                   *type_str;
+			gchar                   *required_str;
+			gchar                   *has_default_str;
+			GType                    type;
+			GossipAccountParamFlags  flags = 0;
+
+			param_name = xmlGetProp (child, "name");
+			type_str = xmlGetProp (child, "type");
+			required_str = xmlGetProp (child, "required");
+			has_default_str = xmlGetProp (child, "has-default");
+
+			if (strcmp (required_str, "yes") == 0) {
+				flags |= GOSSIP_ACCOUNT_PARAM_FLAG_REQUIRED;
 			}
+			if (strcmp (has_default_str, "yes") == 0) {
+				flags |= GOSSIP_ACCOUNT_PARAM_FLAG_HAS_DEFAULT;
+			}
+
+			type = gossip_dbus_type_to_g_type (type_str);
+			g_value = gossip_string_to_g_value (str, type);
+			
+			gossip_account_param_set_full (account, 
+						       param_name,
+						       g_value,
+						       flags);
+
+			g_value_unset (g_value);
+			g_free (g_value);
+			xmlFree (param_name);
+			xmlFree (type_str);
+			xmlFree (required_str);
+			xmlFree (has_default_str);
 		}
-		else if (strcmp (tag, "name") == 0) {
-			name = g_strdup (str);
-		}
+		/* Those are deprecated and kept for compatibility only */
 		else if (strcmp (tag, "id") == 0) {
-			id = g_strdup (str);
-		}
-		else if (strcmp (tag, "password") == 0) {
-			password = g_strdup (str);
-		}
-		else if (strcmp (tag, "server") == 0) {
-			server = g_strdup (str);
+			gossip_account_set_id (account, str);
 		}
 		else if (strcmp (tag, "resource") == 0) {
-			resource = g_strdup (str);
+			gossip_account_set_resource (account, str);
+		}
+		else if (strcmp (tag, "password") == 0) {
+			gossip_account_set_password (account, str);
+		}
+		else if (strcmp (tag, "server") == 0) {
+			gossip_account_set_server (account, str);
 		}
 		else if (strcmp (tag, "port") == 0) {
 			guint tmp_port;
 			
 			tmp_port = atoi (str);
 			if (tmp_port != 0) {
-				port = tmp_port;
-			}
-		}
-		else if (strcmp (tag, "auto_connect") == 0) {
-			if (strcmp (str, "yes") == 0) {
-				auto_connect = TRUE;
-			} else {
-				auto_connect = FALSE;
+				gossip_account_set_port (account, tmp_port);
 			}
 		}
 		else if (strcmp (tag, "use_ssl") == 0) {
-			if (strcmp (str, "yes") == 0) {
-				use_ssl = TRUE;
-			} else {
-				use_ssl = FALSE;
-			}
-		}
-		else if (strcmp (tag, "use_proxy") == 0) {
-			if (strcmp (str, "yes") == 0) {
-				use_proxy = TRUE;
-			} else {
-				use_proxy = FALSE;
-			}
+			gossip_account_set_use_ssl (account, strcmp (str, "yes") == 0);
 		}
 
 		xmlFree (str);
 
 		child = child->next;
 	}
-
-	if (name && id) {
-		account = g_object_new (GOSSIP_TYPE_ACCOUNT, 
-					"type", type,
-					"name", name,
-					"id", id,
-					"port", port,
-					"auto_connect", auto_connect,
-					"use_ssl", use_ssl,
-					"use_proxy", use_proxy,
-					NULL);
-		
-		if (resource) {
-			gossip_account_set_resource (account, resource);
-		}
-
-		if (server) {
-			gossip_account_set_server (account, server);
-		}
-		
-		if (password) {
-			gossip_account_set_password (account, password);
-		}
-		
 #ifdef RESOURCE_HACK
-		const gchar *resource_found = NULL;
+	const gchar *resource_found = NULL;
 
-		if (id) {
-			/* FIXME: This hack is so we don't get bug
-			 * reports and basically gets the resource
-			 * from the id.
-			 */
-			const gchar *ch;
-			
-			ch = strchr (id, '/');
-			if (ch) {
-				resource_found = (ch + 1);
-				id[ch - id] = '\0';
+	str = g_strdup (gossip_account_get_id (account));
+	if (str) {
+		/* FIXME: This hack is so we don't get bug
+		 * reports and basically gets the resource
+		 * from the id.
+		 */
+		const gchar *ch;
 
-				g_printerr ("Converting ID... (id:'%s', resource:'%s')\n", 
-					    id, resource_found);
-			}
+		ch = strchr (str, '/');
+		if (ch) {
+			resource_found = (ch + 1);
+			str[ch - str] = '\0';
+
+			g_printerr ("Converting ID... (id:'%s', resource:'%s')\n", 
+				    str, resource_found);
 		}
+	}
 
-		if (resource_found) {
-			gossip_account_set_id (account, id);
-			gossip_account_set_resource (account, resource_found);
-			need_saving = TRUE;
-		}
+	if (resource_found) {
+		gossip_account_set_id (account, str);
+		gossip_account_set_resource (account, resource_found);
+		need_saving = TRUE;
+	}
+
+	g_free (str);
 #endif /* RESOURCE_HACK */
 
-		gossip_account_manager_add (manager, account);
-		
-		g_object_unref (account);
-	}
-	
-	g_free (name);
-	g_free (id);
-	g_free (password);
-	g_free (resource);
-	g_free (server);
+	gossip_account_manager_add (manager, account);
+
+	g_object_unref (account);
 }
 
 static gboolean
@@ -597,7 +592,9 @@ account_manager_file_parse (GossipAccountManager *manager,
 
 	priv = GET_PRIV (manager);
 	
-	DEBUG_MSG (("Account Manager: Attempting to parse file:'%s'...", filename));
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Attempting to parse file:'%s'...", 
+		      filename);
 
  	ctxt = xmlNewParserCtxt ();
 
@@ -637,11 +634,13 @@ account_manager_file_parse (GossipAccountManager *manager,
 		node = node->next;
 	}
 	
-	DEBUG_MSG (("Account Manager: Parsed %d accounts", 
-		   g_list_length (priv->accounts)));
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Parsed %d accounts", 
+		      g_list_length (priv->accounts));
 
-	DEBUG_MSG (("Account Manager: Default account is:'%s'", 
-		   priv->default_name));
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Default account is:'%s'", 
+		      priv->default_name);
 	
 	xmlFreeDoc(doc);
 	xmlFreeParserCtxt (ctxt);
@@ -661,11 +660,9 @@ account_manager_file_save (GossipAccountManager *manager)
 {
 	GossipAccountManagerPriv *priv;
 	xmlDocPtr                 doc;  
-	xmlDtdPtr                 dtd;  
 	xmlNodePtr                root;
 	GList                    *accounts;
 	GList                    *l;
-	gchar                    *dtd_file;
 	gchar                    *xml_dir;
 	gchar                    *xml_file;
 	mode_t                    old_mask;
@@ -676,22 +673,16 @@ account_manager_file_save (GossipAccountManager *manager)
 		xml_file = g_strdup (priv->accounts_file_name);
 	} else {
 		xml_dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
-		if (!g_file_test (xml_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-			g_mkdir_with_parents (xml_dir, S_IRUSR | S_IWUSR | S_IXUSR);
-		}
+		g_mkdir_with_parents (xml_dir, S_IRUSR | S_IWUSR | S_IXUSR);
 					 
 		xml_file = g_build_filename (xml_dir, ACCOUNTS_XML_FILENAME, NULL);
 		g_free (xml_dir);
 	}
 
-	dtd_file = g_build_filename (DTDDIR, ACCOUNTS_DTD_FILENAME, NULL);
-
 	doc = xmlNewDoc ("1.0");
 	root = xmlNewNode (NULL, "accounts");
 	xmlDocSetRootElement (doc, root);
 
-	dtd = xmlCreateIntSubset (doc, "accounts", NULL, dtd_file);
-
 	if (!priv->default_name) {
 		priv->default_name = g_strdup ("Default");
 	}
@@ -703,40 +694,26 @@ account_manager_file_save (GossipAccountManager *manager)
 	accounts = gossip_account_manager_get_accounts (manager);
 
 	for (l = accounts; l; l = l->next) {
-		GossipAccount *account;
-		gchar         *type, *port;
-		xmlNodePtr     node;
-	
+		GossipAccount           *account;
+		const gchar             *type;
+		xmlNodePtr               node;
+
 		account = l->data;
 
-		switch (gossip_account_get_type (account)) {
-		case GOSSIP_ACCOUNT_TYPE_JABBER:
-		default:
-			type = g_strdup_printf ("jabber");
-			break;
-		}
-	
-		port = g_strdup_printf ("%d", gossip_account_get_port (account));
+		type = gossip_account_type_to_string (gossip_account_get_type (account));
 
 		node = xmlNewChild (root, NULL, "account", NULL);
-		xmlNewChild (node, NULL, "type", type);
+		xmlNewTextChild (node, NULL, "type", type);
 		xmlNewTextChild (node, NULL, "name", gossip_account_get_name (account));
-		xmlNewTextChild (node, NULL, "id", gossip_account_get_id (account));
+		xmlNewTextChild (node, NULL, "auto_connect", gossip_account_get_auto_connect (account) ? "yes" : "no");
+		xmlNewTextChild (node, NULL, "use_proxy", gossip_account_get_use_proxy (account) ? "yes" : "no");
 
-		xmlNewTextChild (node, NULL, "resource", gossip_account_get_resource (account));
-		xmlNewTextChild (node, NULL, "password", gossip_account_get_password (account));
-		xmlNewTextChild (node, NULL, "server", gossip_account_get_server (account));
-		xmlNewChild (node, NULL, "port", port);
-
-		xmlNewChild (node, NULL, "auto_connect", gossip_account_get_auto_connect (account) ? "yes" : "no");
-		xmlNewChild (node, NULL, "use_ssl", gossip_account_get_use_ssl (account) ? "yes" : "no");
-		xmlNewChild (node, NULL, "use_proxy", gossip_account_get_use_proxy (account) ? "yes" : "no");
-
-		g_free (type);
-		g_free (port);
+		gossip_account_param_foreach (account,
+					      (GossipAccountParamFunc) account_manager_add_parameters_foreach,
+					      node);
 	}
 
-	DEBUG_MSG (("Account Manager: Saving file:'%s'", xml_file));
+	gossip_debug (DEBUG_DOMAIN, "Saving file:'%s'", xml_file);
 
 	/* Set the umask to get the proper permissions when libxml saves the
 	 * file, but also change the permissions expiicitly in case the file
@@ -759,4 +736,27 @@ account_manager_file_save (GossipAccountManager *manager)
 	return TRUE;
 }
 
+static void
+account_manager_add_parameters_foreach (GossipAccount      *account,
+					gchar              *param_name,
+					GossipAccountParam *param,
+					xmlNodePtr          node)
+{
+	gchar       *value_str;
+	const gchar *type_str;
+	gboolean     required;
+	gboolean     has_default;
 
+	value_str = gossip_g_value_to_string (&param->g_value);
+	type_str = gossip_g_type_to_dbus_type (G_VALUE_TYPE (&param->g_value));
+	required = param->flags & GOSSIP_ACCOUNT_PARAM_FLAG_REQUIRED;
+	has_default = param->flags & GOSSIP_ACCOUNT_PARAM_FLAG_HAS_DEFAULT;
+
+	node = xmlNewTextChild (node, NULL, "parameter", value_str);
+	xmlNewProp (node, "name", param_name);
+	xmlNewProp (node, "type", type_str);
+	xmlNewProp (node, "required", required ? "yes" : "no");
+	xmlNewProp (node, "has-default", has_default ? "yes" : "no");
+
+	g_free (value_str);
+}
