@@ -65,13 +65,18 @@ jabber_vcard_get_cb (LmMessageHandler   *handler,
 			code = str ? atoi (str) : 0;
 			
 			switch (code) {
-			case 404: {
+			case 404:
 				/* Receipient unavailable */
 				gossip_debug (DEBUG_DOMAIN, "Receipient is unavailable");
 				result = GOSSIP_RESULT_ERROR_UNAVAILABLE;
 				break;
-			}
 
+			case 503:
+				/* Service unavailable */
+				gossip_debug (DEBUG_DOMAIN, "Service is unavailable");
+				result = GOSSIP_RESULT_ERROR_UNAVAILABLE;
+				break;
+				
 			default:
 				gossip_debug (DEBUG_DOMAIN, "Unhandled presence error:%d", code);
 				result = GOSSIP_RESULT_ERROR_INVALID_REPLY;
@@ -111,18 +116,10 @@ jabber_vcard_get_cb (LmMessageHandler   *handler,
 
 	node = lm_message_node_get_child (vcard_node, "EMAIL");
 	if (node) {
-		gchar *email = NULL;
-
-		if (node->value) {
-			email = node->value;
-		} else {
-			node = lm_message_node_get_child (node, "USERID");
-			if (node) {
-				email = node->value;
-			}
+		node = lm_message_node_get_child (node, "USERID");
+		if (node && node->value) {
+			gossip_vcard_set_email (vcard, node->value);
 		}
-			
-		gossip_vcard_set_email (vcard, email);
 	}
 
 	node = lm_message_node_get_child (vcard_node, "URL");
@@ -240,6 +237,7 @@ gossip_jabber_vcard_set (GossipJabber          *jabber,
 	LmConnection       *connection;
 	LmMessage          *m;
 	LmMessageNode      *node;
+	LmMessageNode      *child;
 	LmMessageHandler   *handler;
 	GossipCallbackData *data;
 	gboolean            result;
@@ -259,8 +257,10 @@ gossip_jabber_vcard_set (GossipJabber          *jabber,
 	lm_message_node_add_child (node, "NICKNAME", 
 				   gossip_vcard_get_nickname (vcard));
 	lm_message_node_add_child (node, "URL", gossip_vcard_get_url (vcard));
-	lm_message_node_add_child (node, "EMAIL",
-				   gossip_vcard_get_email (vcard));
+
+	child = lm_message_node_add_child (node, "EMAIL", NULL);
+	lm_message_node_add_child (child, "USERID", gossip_vcard_get_email (vcard));
+
 	lm_message_node_add_child (node, "DESC", 
 				   gossip_vcard_get_description (vcard)); 
 				   
