@@ -70,6 +70,9 @@ static void     presence_chooser_show_dialog            (GossipPresenceChooser *
 							 GossipPresenceState    state);
 static void     presence_chooser_custom_activate_cb     (GtkWidget             *item,
 							 GossipPresenceChooser *chooser);
+static void     presence_chooser_clear_response_cb      (GtkWidget             *widget,
+							 gint                   response,
+							 gpointer               user_data);
 static void     presence_chooser_clear_activate_cb      (GtkWidget             *item,
 							 GossipPresenceChooser *chooser);
 static void     presence_chooser_menu_add_item          (GossipPresenceChooser *chooser,
@@ -145,9 +148,9 @@ gossip_presence_chooser_init (GossipPresenceChooser *chooser)
 	alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
 	gtk_widget_show (alignment);
  	gtk_container_add (GTK_CONTAINER (chooser), alignment); 
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 2, 2);
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 1, 0);
 
- 	priv->hbox = gtk_hbox_new (FALSE, 2); 
+ 	priv->hbox = gtk_hbox_new (FALSE, 1); 
 	gtk_widget_show (priv->hbox);
  	gtk_container_add (GTK_CONTAINER (alignment), priv->hbox); 
 
@@ -160,11 +163,11 @@ gossip_presence_chooser_init (GossipPresenceChooser *chooser)
 	gtk_box_pack_start (GTK_BOX (priv->hbox), priv->label, TRUE, TRUE, 0);
 	gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_END);
  	gtk_misc_set_alignment (GTK_MISC (priv->label), 0, 0.5); 
+ 	gtk_misc_set_padding (GTK_MISC (priv->label), 4, 1); 
 
 	alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
 	gtk_widget_show (alignment);
 	gtk_box_pack_start (GTK_BOX (priv->hbox), alignment, FALSE, FALSE, 0);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 4, 0);
 
 	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
 	gtk_widget_show (arrow);
@@ -252,24 +255,48 @@ presence_chooser_dialog_response_cb (GtkWidget             *dialog,
 		GtkWidget           *entry;
 		GtkWidget           *checkbutton;
 		GtkListStore        *store;
+		GtkTreeModel        *model;
 		GtkTreeIter          iter;
 		GossipPresenceState  state;
 		const gchar         *status;
 		gboolean             save;
+		gboolean             duplicate = FALSE;
+		gboolean             has_next;
 
 		entry = g_object_get_data (G_OBJECT (dialog), "entry");
 		status = gtk_entry_get_text (GTK_ENTRY (entry));
-
 		store = g_object_get_data (G_OBJECT (dialog), "store");
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter, 0, status, -1);
+		model = GTK_TREE_MODEL (store);
 
-		state = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "state"));
+		has_next = gtk_tree_model_get_iter_first (model, &iter);
+		while (has_next) {
+			gchar *str;
 
-		checkbutton = g_object_get_data (G_OBJECT (dialog), "checkbutton");
-		save = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton));
+			gtk_tree_model_get (model, &iter,
+					    0, &str,
+					    -1);
 
-		presence_chooser_set_state (chooser, state, status, save);
+			if (strcmp (status, str) == 0) {
+				g_free (str);
+				duplicate = TRUE;
+				break;
+			}
+
+			g_free (str);
+
+			has_next = gtk_tree_model_iter_next (model, &iter);
+		}
+
+		if (!duplicate) {
+			gtk_list_store_append (store, &iter);
+			gtk_list_store_set (store, &iter, 0, status, -1);
+
+			checkbutton = g_object_get_data (G_OBJECT (dialog), "checkbutton");
+			save = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton));
+			state = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "state"));
+
+			presence_chooser_set_state (chooser, state, status, save);
+		}
 	}
 
 	gtk_widget_destroy (dialog);
@@ -322,7 +349,18 @@ presence_chooser_show_dialog (GossipPresenceChooser *chooser,
 	g_object_unref (pixbuf);
 
 	if (!store[state]) {
+		GList       *presets, *l;
+		GtkTreeIter  iter;
+
 		store[state] = gtk_list_store_new (1, G_TYPE_STRING);
+
+		presets = gossip_status_presets_get (state, -1);
+		for (l = presets; l; l = l->next) {
+			gtk_list_store_append (store[state], &iter);
+			gtk_list_store_set (store[state], &iter, 0, l->data, -1);
+		}
+
+		g_list_free (presets);
 	}
 
 	default_status = gossip_presence_state_get_default_status (state);
@@ -381,19 +419,45 @@ presence_chooser_noncustom_activate_cb (GtkWidget             *item,
 	g_signal_emit (chooser, signals[CHANGED], 0, state, status);
 }
 
+static void
+presence_chooser_clear_response_cb (GtkWidget *widget,
+				    gint       response,
+				    gpointer   user_data)
+{
+	if (response == GTK_RESPONSE_OK) {
+		gossip_status_presets_reset ();
+	}
+
+	gtk_widget_destroy (widget);
+}
+
 static void
 presence_chooser_clear_activate_cb (GtkWidget             *item,
 				    GossipPresenceChooser *chooser)
 {
 	GtkWidget *dialog;
+	GtkWidget *toplevel;
+	GtkWindow *parent = NULL;
 
-	dialog = gtk_message_dialog_new (
-		GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (chooser))),
-		GTK_DIALOG_DESTROY_WITH_PARENT,
-		GTK_MESSAGE_QUESTION,
-		GTK_BUTTONS_NONE,
-		_("Are you sure you want to empy the "
-		  "list?"));
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (chooser));
+	if (GTK_WIDGET_TOPLEVEL (toplevel) &&
+	    GTK_IS_WINDOW (toplevel)) {
+		GtkWindow *window;
+		gboolean   visible;
+
+		window = GTK_WINDOW (toplevel);
+		visible = gossip_window_get_is_visible (window);
+
+		if (visible) {
+			parent = window;
+		}
+	}
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+					 0, 
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_NONE,
+					 _("Are you sure you want to clear the list?"));
 
 	gtk_message_dialog_format_secondary_text (
 		GTK_MESSAGE_DIALOG (dialog),
@@ -405,11 +469,13 @@ presence_chooser_clear_activate_cb (GtkWidget             *item,
 				_("Clear List"), GTK_RESPONSE_OK,
 				NULL);
 
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
-		gossip_status_presets_reset ();
-	}
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
 
-	gtk_widget_destroy (dialog);
+	g_signal_connect (dialog, "response", 
+			  G_CALLBACK (presence_chooser_clear_response_cb),
+			  NULL);
+
+	gtk_widget_show (dialog);
 }
 
 static void
