@@ -84,10 +84,7 @@ typedef struct {
 	GtkWidget *avatar_chooser;
 
 	guint      timeout_id; 
-	guint      saved_id;
-
 	gboolean   requesting_vcard;
-
 	gint       last_account_selected;
 
 #ifdef HAVE_GNOME
@@ -95,36 +92,37 @@ typedef struct {
 #endif
 } GossipVCardDialog;
 
-static void     vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
-							 gint               response,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_avatar_clicked_cb          (GtkWidget         *button,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_avatar_update_preview_cb   (GtkFileChooser    *chooser,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_set_account_to_last        (GossipVCardDialog *dialog);
-static void     vcard_dialog_lookup_start               (GossipVCardDialog *dialog);
-static void     vcard_dialog_lookup_stop                (GossipVCardDialog *dialog);
-static void     vcard_dialog_get_vcard_cb               (GossipResult       result,
-							 GossipVCard       *vcard,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_set_vcard                  (GossipVCardDialog *dialog);
-static void     vcard_dialog_set_vcard_cb               (GossipResult       result,
-							 GossipVCardDialog *dialog);
-static gboolean vcard_dialog_timeout_cb                 (GossipVCardDialog *dialog);
-static void     vcard_dialog_account_changed_cb         (GtkWidget         *combo_box,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_response_cb                (GtkDialog         *widget,
-							 gint               response,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_destroy_cb                 (GtkWidget         *widget,
-							 GossipVCardDialog *dialog);
+static GtkWidget *vcard_dialog_create_avatar_chooser      (void);
+static void       vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
+							   gint               response,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_avatar_clicked_cb          (GtkWidget         *button,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_avatar_update_preview_cb   (GtkFileChooser    *chooser,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_set_account_to_last        (GossipVCardDialog *dialog);
+static void       vcard_dialog_lookup_start               (GossipVCardDialog *dialog);
+static void       vcard_dialog_lookup_stop                (GossipVCardDialog *dialog);
+static void       vcard_dialog_get_vcard_cb               (GossipResult       result,
+							   GossipVCard       *vcard,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_set_vcard                  (GossipVCardDialog *dialog);
+static void       vcard_dialog_set_vcard_cb               (GossipResult       result,
+							   gpointer           user_data);
+static gboolean   vcard_dialog_timeout_cb                 (GossipVCardDialog *dialog);
+static void       vcard_dialog_account_changed_cb         (GtkWidget         *combo_box,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_response_cb                (GtkDialog         *widget,
+							   gint               response,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_destroy_cb                 (GtkWidget         *widget,
+							   GossipVCardDialog *dialog);
 
-/* Called from Glade, so it shouldn't be static. */
-GtkWidget *vcard_dialog_create_avatar_chooser (gpointer data);
 
-GtkWidget *
-vcard_dialog_create_avatar_chooser (gpointer data)
+static GossipVCardDialog *dialog = NULL;
+
+static GtkWidget *
+vcard_dialog_create_avatar_chooser (void)
 {
 	GtkWidget *chooser;
 
@@ -134,12 +132,12 @@ vcard_dialog_create_avatar_chooser (gpointer data)
 						 AVATAR_MAX_WIDTH, 
 						 AVATAR_MAX_HEIGHT);
 
-	gtk_widget_show_all (chooser);
-
 	gtk_widget_set_size_request (chooser, 
 				     AVATAR_MAX_WIDTH / 2,
 				     AVATAR_MAX_HEIGHT / 2);
 
+	gtk_widget_show_all (chooser);
+
 	return chooser;
 }
 
@@ -153,8 +151,8 @@ vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
 		gchar *path;
 
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
-		gossip_image_chooser_set_from_file (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
-						    filename);
+		gossip_image_chooser_set_from_file 
+			(GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser), filename);
 		g_free (filename);
 
 		path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (widget));
@@ -165,8 +163,8 @@ vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
 			g_free (path);
 		}
 	} else if (response == GTK_RESPONSE_NO) {
-		gossip_image_chooser_set_image_data (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
-						     NULL, 0);
+		gossip_image_chooser_set_image_data 
+			(GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser), NULL, 0);
 	}
 
 	gtk_widget_destroy (widget);
@@ -355,17 +353,12 @@ vcard_dialog_lookup_stop (GossipVCardDialog *dialog)
 		g_source_remove (dialog->timeout_id);
 		dialog->timeout_id = 0;
 	}
-
-	if (dialog->saved_id != 0) {
-		g_source_remove (dialog->saved_id);
-		dialog->saved_id = 0;
-	}
 }
 
 static void
 vcard_dialog_get_vcard_cb (GossipResult       result,
 			   GossipVCard       *vcard,
-			   GossipVCardDialog *dialog)
+			   GossipVCardDialog *user_data)
 {
 	GtkComboBox   *combo_box;
 	GtkTextBuffer *buffer;
@@ -373,12 +366,16 @@ vcard_dialog_get_vcard_cb (GossipResult       result,
 	const guchar  *avatar;
 	gsize          avatar_size;
 
-	gossip_debug (DEBUG_DOMAIN, "Got a VCard response");
+	gossip_debug (DEBUG_DOMAIN, "Received VCard response");
+
+	if (!dialog) {
+		return;
+	}
 
 	vcard_dialog_lookup_stop (dialog);
 
 	if (result != GOSSIP_RESULT_OK) {
-		gossip_debug (DEBUG_DOMAIN, "VCard result was not good");
+		gossip_debug (DEBUG_DOMAIN, "Received VCard response was not good");
 		return;
 	}
 
@@ -399,6 +396,7 @@ vcard_dialog_get_vcard_cb (GossipResult       result,
 	gtk_text_buffer_set_text (buffer, STRING_EMPTY (str) ? "" : str, -1);
 
 	avatar = gossip_vcard_get_avatar (vcard, &avatar_size);
+
 	gossip_image_chooser_set_image_data (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
 					     (gchar*) avatar, avatar_size);
 
@@ -407,6 +405,19 @@ vcard_dialog_get_vcard_cb (GossipResult       result,
 	dialog->last_account_selected = gtk_combo_box_get_active (combo_box);
 }
 
+static void
+vcard_dialog_set_vcard_cb (GossipResult result, 
+			   gpointer     user_data)
+{
+	gossip_debug (DEBUG_DOMAIN, "Received VCard response");
+
+	if (!dialog) {
+		return;
+	}
+	
+	gtk_widget_destroy (dialog->dialog);
+}
+
 static void
 vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 {
@@ -460,7 +471,7 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 				  account,
 				  vcard, 
 				  (GossipResultCallback) vcard_dialog_set_vcard_cb,
-				  dialog, &error);
+				  NULL, &error);
 
 	protocol = gossip_session_get_protocol (gossip_app_get_session (), account);
 	contact = gossip_protocol_get_own_contact (protocol);
@@ -471,20 +482,17 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 	g_object_unref (account);
 }
 
-static void
-vcard_dialog_set_vcard_cb (GossipResult       result, 
-			   GossipVCardDialog *dialog)
-{
-	gossip_debug (DEBUG_DOMAIN, "Got a VCard response");
-	
-	gtk_widget_destroy (dialog->dialog);
-}
-
 static gboolean
 vcard_dialog_timeout_cb (GossipVCardDialog *dialog)
 {
 	GtkWidget *md;
 
+	gossip_debug (DEBUG_DOMAIN, "Received VCard lookup timeout");
+
+	if (!dialog) {
+		return FALSE;
+	}
+
 	vcard_dialog_lookup_stop (dialog);
 
 	/* Select last successfull account */
@@ -521,6 +529,7 @@ vcard_dialog_response_cb (GtkDialog         *widget,
 {
 	switch (response) {
 	case RESPONSE_SAVE:
+		gtk_widget_set_sensitive (dialog->button_save, FALSE);
 		vcard_dialog_set_vcard (dialog);
 		return;
 
@@ -548,16 +557,21 @@ vcard_dialog_destroy_cb (GtkWidget         *widget,
 	vcard_dialog_lookup_stop (dialog);
 
 	g_free (dialog);
+	dialog = NULL;
 }
 
 void
 gossip_vcard_dialog_show (GtkWindow *parent)
 {
-	GossipSession     *session;
-	GossipVCardDialog *dialog;
-	GladeXML          *glade;
-	GList             *accounts;
-	GtkSizeGroup      *size_group;
+	GossipSession *session;
+	GladeXML      *glade;
+	GList         *accounts;
+	GtkSizeGroup  *size_group;
+
+	if (dialog) {
+		gtk_window_present (GTK_WINDOW (dialog->dialog));
+		return;
+	}
 
 	dialog = g_new0 (GossipVCardDialog, 1);
 
@@ -581,7 +595,6 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 				       "button_cancel", &dialog->button_cancel,
 				       "button_save", &dialog->button_save,
 				       "button_image", &dialog->button_image,
-				       "avatar_chooser", &dialog->avatar_chooser,
 				       NULL);
 
 	gossip_glade_connect (glade, 
@@ -590,8 +603,14 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 			      "vcard_dialog", "response", vcard_dialog_response_cb,
 			      NULL);
 
+	g_object_add_weak_pointer (G_OBJECT (dialog->dialog), (gpointer) &dialog);
+
 	g_object_unref (glade);
 
+	dialog->avatar_chooser = vcard_dialog_create_avatar_chooser ();
+	gtk_container_add (GTK_CONTAINER (dialog->button_image), dialog->avatar_chooser);
+	
+	/* Line up widgets */
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
 	gtk_size_group_add_widget (size_group, dialog->label_account);
@@ -631,9 +650,10 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 		gtk_widget_hide (dialog->hbox_account);
 	}
 
-	g_list_foreach (accounts, (GFunc)g_object_unref, NULL);
+	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
 	g_list_free (accounts);
 
+	/* Set up transient parent */
 	if (parent) {
 		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), parent); 
 	}
@@ -643,3 +663,4 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 	vcard_dialog_lookup_start (dialog);
 }
 
+
